var acorn = require('acorn');
var _ = require('lodash');
var escodegen = require('escodegen');

function getExportsNode(nodes) {
  return _.find(nodes, function(node) {
    var exp = node.expression
    if (exp && exp.type === 'AssignmentExpression' && exp.left.type === 'MemberExpression' && exp.left.object.name === 'exports' && exp.right.type === 'ObjectExpression') {
      return exp.right;
    }
  })
}

function addToParent(oldKey, property, parent, parentKey, isIdentifier) {
  var newKey;
  if (_.startsWith(oldKey, '&')) {
    newKey = oldKey.split('&').join(parentKey);
  } else {
    newKey = parentKey + ' ' + oldKey;
  }
  if (isIdentifier) {
    property.key.type = 'Literal';
  }
  property.key.value = newKey.trim();
  parent.properties.push(property);
}

function flat(property, parent, parentKey, isIdentifier, object, isFirst) {
  var oldKey = property.key.value || property.key.name || '';
  if (_.contains(oldKey, ',')) {
    _.each(oldKey.split(','), function(key) {
      key = key.trim();
      addToParent(key, _.cloneDeep(property), parent, parentKey, isIdentifier);
    });
    object.properties = _.without(object.properties, property);
  } else if (!isFirst) {
    addToParent(oldKey, property, parent, parentKey, isIdentifier);
    object.properties = _.without(object.properties, property);
  }
  return parent;
}

function flatten(object, parent, parentKey, first) {
  _.each(object.properties, function(p) {
    var newParent = object;
    if ((p.key.type === 'Literal' || p.key.type === 'Identifier') && (p.value.type === 'ObjectExpression' || (p.value.type === 'CallExpression' && p.value.callee.property.name === '_extends')) && parent) {
      newParent = flat(p, parent, parentKey, p.key.type === 'Identifier', object, first);
    }

    if (p.value.type === 'ObjectExpression') {
      flatten(p.value, newParent, p.key.value || p.key.name);
    } else if (p.value.type === 'CallExpression') {
      _.each(p.value.arguments, function(a) {
        if (a.type === 'ObjectExpression') {
          flatten(a, newParent, p.key.value || p.key.name);
        }
      });
    }
  });
  object.properties = _.filter(object.properties, function(p) {
    if (p.value && p.value.properties && _.isEmpty(p.value.properties)) {
      return false;
    }
    return true;
  })
}

module.exports = function(content, map) {
  if (this.cacheable) {
    this.cacheable();
  }

  var tree = acorn.parse(content, {
    ecmaVersion: 6,
    sourceType: 'module'
  });
  var exportNode = getExportsNode(tree.body);
  var parent = exportNode.expression.right;
  flatten(exportNode.expression.right, exportNode.expression.right, '', true);
  return escodegen.generate(tree);
}
